package net.sf.itcb.addons.monitoring.track.soap;

import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;

import net.sf.itcb.addons.monitoring.track.MonitoringTransactionManager;
import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;
import net.sf.itcb.addons.monitoring.track.bean.LayerType;
import net.sf.itcb.common.business.core.ItcbConstants;

import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.ws.client.WebServiceClientException;
import org.springframework.ws.client.core.WebServiceTemplate;

import net.sf.itcb.common.client.aop.ClientInterceptorWebserviceTemplateAware;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.soap.SoapMessage;

/**
 * Client interceptor that put transactionId in soap headers.<br/>
 * It is needed in order to give transactionId to the server.<br/>
 * 
 * <i>Example Configuration in spring context file :</i><br/>
 * <code>
 *   &lt;bean id="yourCompanyWebserviceTemplate" class="org.springframework.ws.client.core.WebServiceTemplate" scope="prototype"&gt;<br/>
 *       &lt;property name="interceptors"&gt;<br/>
 *		    &lt;list&gt;<br/>
 *		      &lt;!-- <ref local="wsSecurityInterceptor" /&gt;--&gt;<br/>
 *		      &lt;ref bean="trackerClientInterceptor" /&gt;<br/>
 *		    &lt;/list&gt;<br/>
 *		&lt;/property&gt;<br/>
 *		&lt;property name="faultMessageResolver" ref="itcbFaultMessageResolver"/&gt;<br/>
 *   &lt;/bean&gt;<br/>
 *  <br/>
 * 
 * &lt;bean id="trackerClientInterceptor" class="net.sf.itcb.addons.monitoring.track.client.TrackerClientInterceptor"/&gt;
 * </code>
 * <br/><br/>
 * 
 * This configuration should be done in company client common spring files in order to be shared by all you IT webservice clients.
 * @author Pierre Le Roux
 *
 */
public class TrackerClientInterceptor implements ClientInterceptorWebserviceTemplateAware {
	

	protected MonitoringTransactionManager monitoringTransactionManager;
	
	protected boolean trackLayer=true;
	
	protected boolean sendTrackerHeader=true; // Set it to false if you call remote webservice that doesn't manage monitoring headers
	
	protected String url;
	
	@Required
	public void setMonitoringTransactionManager(MonitoringTransactionManager monitoringTransactionManager) {
		this.monitoringTransactionManager=monitoringTransactionManager;
	}
	
	public void setTrackLayer(boolean trackLayer) {
		this.trackLayer=trackLayer;
	}
	
	public void setSendTrackerHeader(boolean sendTrackerHeader) {
		this.sendTrackerHeader=sendTrackerHeader;
	}
	
	/**
	 * Store the transaction ID in soap headers
	 */
	@Override
	public boolean handleRequest(MessageContext messageContext)
			throws WebServiceClientException {
		ItcbTrackBean itcbTrackBean = null;
	    
	    
	    if(trackLayer) {
			itcbTrackBean = new ItcbTrackBean();
			long startTime = System.currentTimeMillis();
		    itcbTrackBean.setStartTime(startTime);
		    itcbTrackBean.setLayerType(LayerType.SOAPCLIENT);

		    monitoringTransactionManager.setCurrentTrackBean(itcbTrackBean);
		
	    }
	    else {
	    	itcbTrackBean= monitoringTransactionManager.getCurrentTrackBean();
	    }
	    
	    if(sendTrackerHeader) {
	    	((SoapMessage) messageContext.getRequest()).getSoapHeader().addAttribute(Globals.HEADER_TRANSACTION_ID_QNAME , itcbTrackBean.getTransactionId());
	    	((SoapMessage) messageContext.getRequest()).getSoapHeader().addAttribute(Globals.HEADER_PHASE_ID_QNAME , itcbTrackBean.getCompletePhaseId());
	    }
	    return true;
	}

	@Override
	public boolean handleResponse(MessageContext messageContext)
			throws WebServiceClientException {
		if(trackLayer) {
			handle(messageContext, false);
		}
		return true;
	}
	
	@Override
	public boolean handleFault(MessageContext messageContext)
			throws WebServiceClientException {		
		if(trackLayer) {
			handle(messageContext, true);
		}
		return true;
	}
	
	private void handle(MessageContext messageContext, boolean isFault) {
		 
		ItcbTrackBean itcbTrackBean = monitoringTransactionManager.getCurrentTrackBean();
		itcbTrackBean.setClassName(url);
    	SoapMessage soapRequest = ((SoapMessage) messageContext.getRequest());
		 //Retrieving invoked request
        if(soapRequest != null) {
	        Source source = soapRequest.getPayloadSource();
	
	        if (source != null && source instanceof DOMSource) {
	            DOMSource dom = (DOMSource) source;
	            org.w3c.dom.Node node = dom.getNode();
	            itcbTrackBean.setMethod(node.getLocalName());
	            
	            itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - itcbTrackBean.getStartTime());
	            if(isFault) {
	            	
	            	SoapMessage soapResponse = ((SoapMessage) messageContext.getResponse());
	            	itcbTrackBean.setException(new SoapRequestToString(soapResponse));
	            	itcbTrackBean.setParameters(new Object[] {new SoapRequestToString(soapRequest)});
	            }
	            itcbTrackBean.setUser(MDC.get(ItcbConstants.MDC_USER_KEY));
	        }
        }
    	monitoringTransactionManager.endCurrentTrackBean(); 
		 
		
	}

	@Override
	public void setWebserviceTemplate(WebServiceTemplate webServiceTemplate) {
		this.url=webServiceTemplate.getDestinationProvider().getDestination().toString();
		
	}

}
