package net.sf.itcb.addons.monitoring.track.soap;

import javax.xml.transform.Source;
import javax.xml.transform.dom.DOMSource;

import net.sf.itcb.addons.monitoring.track.MonitoringTransactionManager;
import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;
import net.sf.itcb.addons.monitoring.track.bean.LayerType;
import net.sf.itcb.common.business.core.ItcbConstants;

import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.server.EndpointInterceptor;
import org.springframework.ws.soap.SoapHeader;
import org.springframework.ws.soap.SoapMessage;

/**
 * This server interceptor receives the transaction id from the client.<br/>
 * It retrieves it in SoapHeader with {@link Globals#HEADER_TRANSACTION_ID_QNAME} Qname.
 * 
 * This interceptor has to be activated in your server modules spring configuration as in the following example :<br/>
 * <code>
 *   &lt;bean class="org.springframework.ws.server.endpoint.mapping.PayloadRootAnnotationMethodEndpointMapping"&gt;<br/>
 *	    &lt;property name="interceptors"&gt;<br/>
 *		    &lt;list&gt;<br/>
 *            &lt;ref bean="trackerEndpointInterceptor"/&gt;<br/>
 *          ...<br/>
 *		      &lt;ref bean="validatingInterceptor" /&gt;<br/>
 *          ...<br/>
 *		      
 *		    &lt;/list&gt;<br/>
 *		&lt;/property&gt;<br/>
 *   &lt;/bean&gt;<br/><br/>
 *   
 *   &lt;bean id="trackerEndpointInterceptor" class="net.sf.itcb.addons.monitoring.track.server.TrackerEndpointInterceptor"&gt;
 *   	&lt;property name="itcbAppender" ref="xxxxYourAppender"/&gt;
 *   &lt;/bean&gt;
 * </code> 
 * <br/>
 * This declaration has to be done in the webservice config file in your src/main/webapp/WEB-INF/xxxx-servlet.xml
 * @author Pierre Le Roux
 *
 */
public class TrackerEndpointInterceptor implements EndpointInterceptor {

	private MonitoringTransactionManager monitoringTransactionManager;
	
	@Required
	public void setMonitoringTransactionManager(MonitoringTransactionManager monitoringTransactionManager) {
		this.monitoringTransactionManager=monitoringTransactionManager;
	}
	
	@Override
	public boolean handleFault(MessageContext messageContext, Object endpoint)
			throws Exception {
		
		handle(messageContext, endpoint, true);
		return true;
	}

	
	
	@Override
	public boolean handleRequest(MessageContext messageContext, Object endpoint)
			throws Exception {
		SoapMessage soapMessage = ((SoapMessage) messageContext.getRequest());
		SoapHeader soapHeader = soapMessage.getSoapHeader();

		ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		long startTime = System.currentTimeMillis();
	    itcbTrackBean.setStartTime(startTime);
	    itcbTrackBean.setLayerType(LayerType.SOAPSERVER);
		
	    if( soapHeader != null) {
	    	String transactionId = soapHeader.getAttributeValue(Globals.HEADER_TRANSACTION_ID_QNAME);
			String parentPhaseId = soapHeader.getAttributeValue(Globals.HEADER_PHASE_ID_QNAME);
			
			// If transactionId has been set, we store it in the current Thread	
			if(transactionId != null) {
				itcbTrackBean.setTransactionId(transactionId);
			}
			if(parentPhaseId != null) {
				itcbTrackBean.setParentPhaseId(parentPhaseId);
			}
		}
		
		monitoringTransactionManager.setCurrentTrackBean(itcbTrackBean);
		
		return true;
	}

	@Override
	public boolean handleResponse(MessageContext messageContext, Object endpoint)
			throws Exception {
		
		handle(messageContext, endpoint, false);
		return true;
	}
	
	
	
	
	
	private void handle(MessageContext messageContext, Object endpoint, boolean isFault) {
		 
		//ItcbTrackBean itcbTrackBean = (ItcbTrackBean)messageContext.getProperty(PROPERTY_TRACK_BEAN);
		ItcbTrackBean itcbTrackBean = monitoringTransactionManager.getCurrentTrackBean();
		itcbTrackBean.setClassName(endpoint.getClass().getName());
	    	
    	SoapMessage soapRequest = ((SoapMessage) messageContext.getRequest());
		 //récupération de l'opération invoquée
        if(soapRequest != null) {
	        Source source = soapRequest.getPayloadSource();
	
	        if (source != null && source instanceof DOMSource) {
	            DOMSource dom = (DOMSource) source;
	            org.w3c.dom.Node node = dom.getNode();
	            itcbTrackBean.setMethod(node.getLocalName());
	            
	            itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - itcbTrackBean.getStartTime());
	            if(isFault) {
	            	
	            	SoapMessage soapResponse = ((SoapMessage) messageContext.getResponse());
	            	itcbTrackBean.setException(new SoapRequestToString(soapResponse));
	            	itcbTrackBean.setParameters(new Object[] {new SoapRequestToString(soapRequest)});
	            }
	            itcbTrackBean.setUser(MDC.get(ItcbConstants.MDC_USER_KEY));
	        }
        }
    	monitoringTransactionManager.endCurrentTrackBean(); 
		 
		
	}

	
	

	@Override
	public void afterCompletion(MessageContext messageContext, Object endpoint,
			Exception ex) throws Exception {
		
		
	}
}
